#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from reclaimer.field_types import *

H3TagRef      = FieldType(base=TagRef, name="H3TagRef")
H3Reflexive   = FieldType(base=Reflexive,   name="H3Reflexive")
H3RawdataRef  = FieldType(base=RawdataRef,  name="H3RawdataRef")
