#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

# DO NOT REMOVE! __all__ is used to locate frozen definitions
__all__ = (
    "actr", "actv", "ant_", "antr", "bipd", "bitm", "boom", "cdmg", "coll",
    "colo", "cont", "ctrl", "deca", "DeLa", "devc", "devi", "dobc", "effe",
    "elec", "eqip", "flag", "fog_", "font", "foot", "garb", "glw_", "grhi",
    "hmt_", "hud_", "hudg", "item", "itmc", "jpt_", "lens", "lifi", "ligh",
    "mach", "matg", "metr", "mgs2", "mod2", "mode", "mply", "ngpr", "obje",
    "part", "pctl", "phys", "plac", "pphy", "proj", "rain", "sbsp", "scen",
    "scex", "schi", "scnr", "senv", "sgla", "shdr", "sky_", "smet", "snd_",
    "snde", "lsnd", "soso", "sotr", "Soul", "spla", "ssce", "str_", "swat",
    "tagc", "trak", "udlg", "unhi", "unit", "ustr", "vcky", "vehi", "weap",
    "wind", "wphi",
    )
